# from EDutils import __version__
from . import __version__

print(__version__)

##too slow
class Shortcut_finder:
    def __init__(self,df_keys):
        self.df_keys=df_keys
        self.fig,self.ax = dsp.stddisp(figsize=(0.3,0.2),pOpt='')#,texts=['type shortcut'])
        cid = self.fig.canvas.mpl_connect('key_press_event', self)
    def __call__(self,event):
        dfk = self.is_key(event.key)#;print(dfk)
        self.ax.cla()
        self.ax.axis('off')
        self.ax.text(0.5,0.5,'Shortcut : %s\n' %event.key     ,color='k',fontsize=30,horizontalalignment='center')
        if any(dfk.index) :
            txt = dfk.index[0].replace('_',' ')
            self.ax.text(0.5,0.25,txt,color='g',fontsize=30,horizontalalignment='center')
        self.fig.canvas.draw()
    def is_key(self,key):return self.df_keys.loc[self.df_keys.key==key]
    def is_cmd(self,cmd):return self.df_keys.iloc[[cmd in k for k in  self.df_keys.index]]



from utils import*
import importlib as imp
import wallpp.lattice as lat        ;imp.reload(lat)
import wallpp.plane_group as pg     ;imp.reload(pg)
import multislice.multi_2D as ms    ;imp.reload(ms)
plt.close('all')
path='../multislice/docs_fig/multi2D/2_beam_'
opts='T' #V(potential), T(tilts/rocking curve), M(single run), E(wald), K()'

K = 5.008343065817388
keV   = cst.lam2keV(1/K)#200       # wavelength 200keV
Nx,Nz = 1,5000     # unit cells
ndeg  = 2**6       # number of pixels
npx   = Nx*ndeg    # number of pixels
pptype,ax,bz,angle = 'p1',2,2,90
Za = 2
pattern = np.array([[1,1,1]])
Nh = 20
rot = lambda t:np.array([[np.cos(t),np.sin(t)],[-np.sin(t),np.cos(t)]])

a1,a2 = lat.get_lattice_vec(lat_type='rect',a=ax,b=bz)
b1,b2 = lat.reciprocal_lattice_2D(a1,a2)
Ai = np.array([0.1,0.25,0.26,0.27,1.5])
fj = lambda ti,j:np.sqrt(np.pi*Ai[j])*np.exp(-(np.pi*Ai[j]*K*np.sin(ti))**2)

n = 10
ax1 = np.sqrt(1**2+n**2)*ax
bz1 = ax1


if 'V' in opts:
    # p1 = pg.Wallpaper(pptype,ax,bz,angle,pattern,ndeg=ndeg)
    # potential = p1.get_potential_grid()
    x0,z0 = np.meshgrid(np.arange(-2,n+1),np.arange(-1,n+2))
    x0,z0 = np.stack([x0*ax+pattern[0][0],z0*bz+pattern[0][1]])
    Xa = rot(1/n).dot(np.stack([x0.flatten(),z0.flatten()]))

    x,z = np.meshgrid(np.arange(n*ndeg)*ax1/(n*ndeg),np.arange(n*ndeg)*bz1/(n*ndeg))
    f = np.sum(np.array([pg.fv(np.stack([x.flatten(),z.flatten()]).T,X0,Za) for X0 in Xa.T]),axis=0)
    f = np.reshape(f,(n*ndeg,n*ndeg))
    np.save('dat/2_beam.npy',[x,z,f])
    dsp.stddisp(im=[x,z,f],scat=[Xa[0],Xa[1],15,'k'],labs=[r'$x(\AA)$',r'$z(\AA)$']
        ,name=path+'fv.png',opt='ps')

else:
    x,z,f = np.load('dat/2_beam.npy')

if 'M' in opts:
    ms0 = ms.Multi2D([x[0],z.T[0],f],ax1,bz1,keV=keV,Nx=Nx,nz=Nz,
            dz=bz1/(2*n),eps=0.01,sg=-1,
            iZs=1,iZv=np.inf)
    ms0.Bz_show(np.arange(-3,3)*n,lw=2  ,name=path+'Iz.svg',opt='p')
    ms0.Qz_show(slice(0,-1,100),opts='O',name=path+'Iq.svg',opt='p',xylims=['x',-5,5])

    # if 'D' in opts:
    #     iM = -n*4
    #     fig,ax = ms0.Bz_show(iM)
    #     m = ms0.psi_qz[:,iM].max()/ms0.nx**2/ms0.dq
    #
    #     fj = lambda ti,j:np.sqrt(np.pi*Ai[j])*np.exp(-(np.pi*Ai[j]*K*np.sin(ti))**2)
    #     pg.fv()
    #
    #     kx = 1#4/ax
    #     ti   = np.arcsin(kx/ms0.k0)
    #     xi_g = np.pi/(ms0.sig*ms0.eps*fj(ti,Za))*ax*bz
    #     w_g  = ms0.k0*(1-np.cos(ti))
    #     F = np.sin(t/xi_g)**2
    #     plts = [[ms0.z,F,'b--']]
    #     dsp.stddisp(plts,ax=ax0,lw=2,labs=[r'$\AA(nm)$','$I$'])

if 'T' in opts:
    tilts=np.linspace(0,0.08,40)
    mst = np.zeros(tilts.size,dtype=object)
    for i,t in enumerate(tilts):
        mst[i] = ms.Multi2D([x[0],z.T[0],f],tilt=t,
            ax=ax1,bz=bz1,keV=keV,Nx=Nx,nz=Nz,
            dz=bz1/(2*n),eps=0.01,sg=-1,
            iZs=10,iZv=np.inf,v=1)
    z   = mst[19].z
    zeta_g = 2925                       #measured Pendullosung thickness
    s_g = np.sin(np.pi*z/zeta_g)**2     #fit
    cg  = dsp.getCs('jet',3)[-1]
    plts=[[z,s_g,[cg,'--'],'']]
    fig,ax=mst[19].Bz_show(np.array([0,10,20]),plts=plts,lw=2,opt='ps',name=path+'Itheta_c.svg')
    # mst[19].Qz_show(opts='O', xylims=['x',-1,2],opt='ps',name=path+'Itheta_q.svg')

    zs  = np.array([0.25,0.5,0.75,1,1.25,1.5])*zeta_g #selected z for rocking curve
    iZs = [np.argmin(abs(z-z0)) for z0 in zs]
    # ms.tilts_show(tilts,mst,iBs=20,iZs=iZs,opt='p',name=path+'rocking.svg')
    # ms.tilts_show(tilts,mst,iBs=10,iZs=slice(0,Nz,100))

if 'E' in opts:
    ms0 = ms.Multi2D([x[0],z.T[0],f],ax1,bz1,keV=keV,Nx=Nx,nz=1,
            dz=bz1/(2*n),eps=0.01,sg=-1,
            iZs=1,iZv=np.inf)

    g,theta = 1,0.1
    # Ks = np.linspace(0,2,100)
    # F = lambda K:K*(1-np.sin(2*np.arcsin(g/(2*K))))-g*np.sin(theta)
    # dsp.stddisp([Ks,F(Ks),'b'],lw=2)
    K  = 1/(2*np.sin(theta))#1/cst.keV2lam(keV)
    # K = cst.keV2A(12)
    dt = 30*np.pi/180 #10deg each side
    t  = 3*np.pi/2+dt*np.linspace(-1,1,1000)

    nh,nk = 20,10
    h,k = np.meshgrid(np.arange(-nh,nh+1),np.arange(nk))
    X = h*b1[0]+k*b2[0]
    Z = h*b1[1]+k*b2[1]
    # ss = X.shape
    X,Z  = rot(-theta).dot(np.stack([X.flatten(),Z.flatten()]))
    # X,Z = X.reshape(ss)
    H = Nz*bz
    zeta = np.linspace(0,0.25/bz,100)
    Fz = lambda i : 1/(1.1*ax)*np.sinc(zeta*H)**2+i/ax

    plts=[]
    # plts +=[[Fz(i),zeta,'b--',''] for i in range(nh)]
    plts += [[K*np.cos(t),K*np.sin(t)+K,'r','']]
    scat  = [X,Z,80,'b']
    ax1 = np.sqrt(1**2+10**2)*ax
    bz1 = ax1

    fig,ax0 = ms0.Ewald_show()
    dsp.stddisp(plts,scat=scat,labs=[r'$k_x(\AA^{-1})$',r'$k_z(\AA^{-1})$'],
        lw=2,xylims=[-0.01,1.1,-0.01,0.2],ax=ax0,name=path+'E.png',opt='ps')#,xyTicks=[1/ax1,1/bz1])


if 'K' in opts:
    z = ms0.z
    K = ms0.k0
    kx = np.arange(1,Nh)/ax
    ti = np.arcsin(kx/K)
    xi = K*(1-np.cos(ti))

    iM = 3
    m = ms0.psi_qz[:,iM+1].max()/ms0.nx**2/ms0.dq
    F = lambda i:fj(ti[i],Za)*np.sin(np.pi*z*xi[i])/(np.pi*xi[i])#*ms0.sig/(ax*bz)
    fig,ax0 = ms0.Bz_show(np.arange(1,Nh)*Nx,lw=2)
    cs = dsp.getCs('jet',Nh-1)
    plts = [[z,(F(i)/F(iM).max())**2*m,[cs[i],'--'],'$%d$' %i] for i in range(Nh-1)]
    dsp.stddisp(plts,ax=ax0,lw=2,labs=[r'$\AA(nm)$','$I$'])
    # plts  = [[t,fj(t,Za)**2,'g--','fe**2']]



#!/usr/bin/env python3

from distutils.core import setup

setup(name='python-markdown-math',
      description='Math extension for Python-Markdown',
      author='Dmitry Shachnev',
      author_email='mitya57@gmail.com',
      version='0.1',
      url='https://github.com/mitya57/python-markdown-math',
      py_modules=['mdx_math'],
      license='BSD')



from utils import*                  ;imp.reload(dsp)
from EDutils import viewers as vw   ;imp.reload(vw)
plt.close('all')



# v = vw.Viewer(path='dat/diamond',cif_file='diamond',u=[0,0,1],pets_opts='L',
#     init_opts='R')
v = vw.Viewer(config='dat/diamond/config.pkl')



from utils import*                  ;imp.reload(dsp)
from blochwave import bloch         ;imp.reload(bloch)
from EDutils import viewers as vw   ;imp.reload(vw)
plt.close('all')
path = 'dat'

b0 = bloch.load_Bloch(path,'Si001')

u010 = np.array([ 0.05929628, -0.00230903,  0.99823776])
u020 = np.array([ 0.13922443, -0.00461806,  0.99025009])
u030 = np.array([ 0.39685357,  0.0069271 ,  0.9178558 ])
u040 = [ 0.26448615, -0.00923606,  0.96434526]
# u = [ 0.2980959 , -0.00520328,  0.95452174]
# u = [ 0.13503907,0.03366902,-0.99026807]
u = None#u040#u010
theta,phi=0,0
u = [2,5,0]
# theta,phi,u=15.3460, 358.0000,None

# b0.solve(Smax=0.01,u=[1,1,0],Nmax=15)
# b0.show_beams_vs_thickness(thicks=(0,1000,1000))

rot_vw = vw.Rotate_Viewer(b0,[theta,phi,1,1],
    u=u,Smax=0.01,Nmax=10,thick=250,
    F='Sw',opts='x',cmap='Greens',mag=100,xylims=3,init='wz')

# b0.is_hkl([0,0,0])
# I = b0.get_intensities()
# b0.get_Istrong(m=1000)
# b0.get_Sw(Smax=0.01)

# b0.set_thickness(100)
# b0.show_beams(F='I')
# b0.show_beams(F='Vg',fopts='m',opts='',pOpt='im',cmap='jet',mag=500)
# b0.show_beams(F='S' ,fopts='m',opts='',pOpt='im',cmap='jet',mag=500)

# h,k,l = b0.get_hkl().T
# u,v,w = b0.Kuvw
# print(u*h+k*v+l*w)


def test_bloch_solve():
    b0 = bloch.Bloch('Si',path=path,keV=200)#,u=[0,0,1],Nmax=3,Smax=0.02,opts='s0')
    b0.solve(u=[0,0,1],Smax=0.01,Nmax=3, opts='s0')
    # b0.solve(u=[1,1,0],Smax=0.01,Nmax=3, opts='s0')
    # b0.solve(u=[1,1,1],Smax=0.01,Nmax=3, opts='s0')

def check_multi_spot():
    ar = b0.df_G[['px','py']].values
    u,idx,idr,count = np.unique(ar,axis=0,return_index=True,return_inverse=True,return_counts=True)

    for u0 in u[count>1]:
        # ar[idx0]
        ids = np.where(np.linalg.norm(ar-u0,axis=1)<1e-3)[0]
        print(b0.df_G.iloc[ids][['h','k','l','px','py','Vg']])
    # rep = np.setdiff1d(np.arange(133),idr[idx])

def test_set_beam(b0):
    b0.set_beam(keV=200)
    b0.set_beam(u=[1,0,1])
    b0.set_beam(K=1/cst.keV2lam(100)*np.array([1,0,0]))
def test_show_Fhkl(b0):
    b0.show_Fhkl(xylims=6,h3D=1,ms=30,cmap='Greens')
    b0.show_Fhkl(s=np.s_[:,:,3],xylims=6,ms=30,cmap='Greens')
    b0.show_Fhkl(s='h=0',opts='l',xylims=7,ms=50,cmap='Greens',pOpt='im',caxis=[0,1.5])

# test_bloch_solve()
# test_set_beam(b0)
# test_show_Fhkl(b0)



import importlib as imp
from utils import*                          ; imp.reload(dsp)
import multislice.multislice as mupy        ; imp.reload(mupy)
import multislice.postprocess as pp         ; imp.reload(pp)
import multislice.mupy_utils as mut         ; imp.reload(mut)
path ='../multislice/dat/Carbone_SC/'
figpath = '../docs_fig/'
plt.close('all')

def run(data,tail, **kwargs):
    multi = mupy.Multislice(path,tail=tail,data=data,
        mulslice=False,keV=200,TDS=False,i_slice=10,
        NxNy=1024,slice_thick=3.0,repeat=[1,1,1],
        opt='dsrp', **kwargs)
    return multi

rotx = lambda a:np.array([[1,0,0],[0,np.cos(a),np.sin(a)],[0,-np.sin(a),np.cos(a)]])
# roty = lambda a:np.array([[np.cos(a),0,np.sin(a)],[0,1,0],[-np.sin(a),0,np.cos(a)]])
# rotz = lambda a:np.array([[0,np.cos(a),np.sin(a),0],[-np.sin(a),np.cos(a),0],[0,0,1]])
#
# lat_params  = np.array([1,2,3])
# lat_rec = 1/lat_params
# abc=np.diag([1,0.5,1/3])
# Rx,Ry = rotx(np.pi/4),rotx(np.pi/3)
# xh = [0,0.5,2/3]
# h = abc.dot(xh)/lat_rec**2
#
# Rabc = Rx.dot(Ry).dot(abc)
# Rxh = Rx.dot(Ry).dot(xh)
# Rh = Rabc.T.dot(Rxh)/lat_rec**2
# print(h,Rh)

def test_001():
    lat_params  = [1,2,1]
    lat_vec     = np.diag(lat_params)
    pattern     = np.array([[6,0.5,0.5,1, 1,1],[8,0.75,0.75,0.5, 1,1]])

    name=path+'carbone001.xyz'
    n = [0,0,1] # beam along z
    u = [1,0,0] # rotation axis along x
    rep,pad = [20,20,10],4
    mut.make_xyz(name,pattern,lat_vec,lat_params,n=n,pad=pad,rep=rep,fmt='%.4f',dopt='s')

    # mut.show_grid(name,'xy')
    # mut.show_grid(name,'xz')
    # mut.show_grid(name,'yz')
    # multi = run(data=name,tail='001',Nhk=31,hk_sym=0,fopt=1,ppopt='wBP')
    multi = pp.load_multi_obj(path+'Carbone_SC_001_autoslic.pkl')
    P = multi.show_patterns(Iopt='Incs',out=False,Nmax=256,
            caxis=[0,0.01],imOpt='cv',axPos='V',cmap='gray')
            # xylims=0.4*np.array([-1,1,-1,1]),name=figpath+'900_I.png',opt='')

    #
    #
    # hs,ks = np.meshgrid(np.arange(0,3)*rep[0]*(1+2*int(pad)),np.arange(0,3)*rep[1]*(1+2*int(pad)))
    # hs,ks = hs.flatten(),ks.flatten()
    # iBs = ['(%d,%d)' %(h,k) for h,k in zip(hs,ks)]
    # multi.beam_vs_thickness(iBs=iBs,cm='Spectral',name=figpath+'C001.svg',opt='p')
    # # hs,ks = np.meshgrid(np.arange(15),np.arange(15))
    # # hs,ks = hs.flatten(),ks.flatten()
    # # iBs = ['(%d, %d)' %(h,k) for h,k in zip(hs,ks)]
    # # multi.beam_vs_thickness(iBs=iBs,cm='Spectral',name=figpath+'C001.svg',opt='p')
    #
    #
    # fig,ax = multi.pattern(Iopt='Incs',out=False,Nmax=256,
    #     #tol=1e-3,
    #     caxis=[0,0.01],
    #     # rings=[0.25,0.5],lw=2,
    #     imOpt='cv',axPos='V',cmap='gray',opt='p')
    #     # xylims=0.4*np.array([-1,1,-1,1]),name=figpath+'900_I.png',opt='')
    #
    # qx,qy,I = multi.pattern(Iopt='Incs',out=True,Nmax=256,opt='c')
    # Nmax = 256
    # s1,s2 = np.s_[Nmax,:],np.s_[:,Nmax]
    #
    # plts=[]
    # plts+=[[np.arange(I[s1].size)-Nmax,I[s1],'r']]
    # plts+=[[np.arange(I[s2].size)-Nmax,I[s2],'b']]
    # # plts = [[qx[s1],I[s1],'r','x'],[qy[s2],I[s2],'b','y']]
    # dsp.stddisp(plts,lw=2)#,xylims=['y',0,0.001])


test_001()



import importlib as imp
from utils import*                      ;imp.reload(dsp)
import multislice.pymultislice as pyms  ;imp.reload(pyms)
import multislice.multi_2D as MS2D      ;imp.reload(MS2D)
import wallpp.lattice as lat            ;imp.reload(lat)
import wallpp.plane_group as pg         ;imp.reload(pg)
import scipy.fftpack as fft

plt.close('all')
path='../../multislice/docs_fig/multi2D/TDS/'
opts='0'

ax,bz = 8,10
pattern = [[2,3,2],[4,6,4],[1.5,2.0,1],[2.5,1.5,1]]
ndeg = 2**10
keV  = 200


Nx,nTDS = 32,32
file = 'dat/TDS_Nx%d_nTDS%d.pkl' %(Nx,nTDS)
if 'r' in opts:
    mp = MS2D.Multi2D(pattern,ax,bz,keV=keV,
            Nx=Nx,dz=2,nz=10,eps=0.25,nx=ndeg,
            TDS=True,nTDS=nTDS,wobble=np.array([0.002,0.025,0.025,0.025,0.05])*2, #[0.01,1:0.025,2:0.025,4:0.05},
            ppopt='',opts='q',v=1)
    mp.save(file)
else :
    mp = pyms.load(file)

mp.Za_show()
mp.Qz_show(iZs=[-1],title='with TDS',xylims=['x',-2,2])

# mp.Bz_show(iBs=np.arange(0,4),sym_opt=1,lw=2)

if '0' in opts:
    # mp = MS2D.Multi2D(pattern,ax,bz,keV=keV,
    #         Nx=Nx,dz=2,nz=10,eps=0.25,nx=ndeg,
    #         TDS=True,nTDS=1,wobble=np.array([0.002,0.025,0.025,0.025,0.05])*0, #[0.01,1:0.025,2:0.025,4:0.05},
    #         ppopt='',opts='q',v=1)

    p1 = pg.Wallpaper('p1',ax,bz,90,pattern,ndeg=ndeg,gen=True,nh=3,nk=3)
    # p1.plot_unit_cells(opts='AV',nh=2,nk=2)
    potential = p1.get_potential_grid_p1()
    #
    # # Loop over 1 unit cell periodic
    mp = MS2D.Multi2D(potential,ax,bz,keV=keV,
            Nx=Nx,dz=2,nz=10,eps=0.25,
            ppopt='',opts='q',v=1)

    # mp.Ewald_show()
    # mp.Tz_show(Vopt='V',lw=2,cmaps='jet')
    mp.Za_show()
    mp.Qz_show(iZs=[-1],xylims=['x',-2,2],title='no TDS')
    # mp.Bz_show(iBs=np.arange(0,4)*mp.Nx,sym_opt=1,lw=2)
    # mp.Bz_show(iBs=np.arange(6),lw=2)



from utils import*                  #;imp.reload(dsp)
from EDutils import viewer_config as vw_cfg   ;imp.reload(vw_cfg)
plt.close('all')

key_map = vw_cfg.KeyBinder()
txt = 'type a key to see the mapping on the console'
fig,ax = dsp.stddisp(pOpt='',texts=[0.5,0.5,txt,'g'],figsize='21')#,[0.5,0.25],)
cid = fig.canvas.mpl_connect('key_press_event', key_map)
fig.show()



'''Bloch wave solver'''
import importlib as imp
import numpy as np,pandas as pd,pickle5,os,glob
from crystals import Crystal
from multislice import mupy_utils as mut     ;imp.reload(mut)
from multislice import postprocess as pp     ;imp.reload(pp)
from scattering import structure_factor as sf;imp.reload(sf)
from utils import displayStandards as dsp    ;imp.reload(dsp)
from utils import physicsConstants as cst    ;imp.reload(cst)
from utils import glob_colors as colors,handler3D as h3d


class Bloch:
    def __init__(self,cif_file,keV=200,K=None,u=[0,0,1],Nmax=1,
        Smax=0.2,thick=500,name='',path='',solve=0,opts='sv'):
        ''' Bloch wave simulation class
        - cif_file : str - structure
        - name     : str - optional name (used to save object see set_name)
        - K,u,keV  : reciprocal space beam vector(see update beam)
        - Nmax     : int - max order of reflections/resolution(see update_Nmax)
        - Smax     : float - maximum excitation error to be included(see solve)
        - thick    : float - thickness of crystal(can be modified after solve)
        - solve    : bool - diagonalize the Bloch matrix and find intensities
        - opts     : str - (see solve)
        '''
        self.cif_file = cif_file
        self.crys     = mut.import_crys(cif_file)
        self.lat_vec0 = np.array(self.crys.lattice_vectors)
        self.lat_vec  = np.array(self.crys.reciprocal_vectors)/(2*np.pi)
        self.pattern  = np.array([np.hstack([a.coords_fractional,a.atomic_number]) for a in self.crys.atoms] )
        self.Nmax  = 0
        self.thick = thick
        self.update_Nmax(Nmax)
        self.set_beam(K,u,keV)
        self.set_name(name,path)
        if solve or 'S' in opts:self.solve(Smax,opts=opts)

    def set_name(self,name='',path=''):
        '''set name for Bloch obj(will be saved as path+name+'.pkl'):
        - name : The name of the simulation(can include the full path)
        - path : The path to the simulation folder
        By default the simulation name is '<cif><zone axis>_<keV>keV_bloch'
        '''
        if not name:
            basefile = self.cif_file.replace('.cif','')
            u_str = ''.join(['%d' %np.round(u) for u in self.Kabc0])
            name='%s%s_%dkeV_bloch' %(basefile,u_str,np.round(self.keV))
        if not path:os.path.dirname(name)
        self.path = path                            #; print(self.path)
        self.name = os.path.basename(name)

    def update_Nmax(self,Nmax):
        '''Update resolution/max order. (Updates lattice and Fhkl)
        Nmax : maximum h,k,l order
        '''
        if isinstance(Nmax,int):
            if not Nmax==self.Nmax:
                self.Nmax=Nmax
                (h,k,l),(qx,qy,qz) = mut.get_lattice(self.lat_vec,self.Nmax)
                self.lattice = [(h,k,l),(qx,qy,qz)]
                self.hklF,self.Fhkl = sf.structure_factor3D(self.pattern, 2*np.pi*self.lat_vec,hklMax=2*self.Nmax)

    def set_beam(self,K=None,u=None,keV=None):
        ''' Update the beam
        - keV : float - update beam wavelength
        - u   : list - update beam direction
        - K   : list - update beam direction and wavelength
        '''
        if keV:self.k0 = 1/cst.keV2lam(keV)
        if isinstance(K,list) or isinstance(K,np.ndarray):
            self.K  = K
            self.k0 = np.linalg.norm(K)
        elif isinstance(u,list) or isinstance(u,np.ndarray):
            self.K = self.k0*np.array(u)/np.linalg.norm(u)
        self.lam = 1/self.k0
        self.keV = cst.lam2keV(self.lam)
        self.sig = cst.keV2sigma(self.keV)
        self.u   = self.K/self.k0
        self.Kuvw = self.lat_vec.dot(self.K)  #projection in reciprocal crystal basis
        self.Kabc = self.lat_vec0.dot(self.K) #projection in crystal basis
        self.Kabc0 = self.Kabc/(abs(self.Kabc)[abs(self.Kabc)>0.01].min()) #;print(Kabc)
        self.Kuvw0 = self.Kuvw/(abs(self.Kuvw)[abs(self.Kuvw)>0.01].min()) #;print(Kabc)

    def set_thickness(self,thick):
        '''set thickness and update beams
        - thick : thickness
        '''
        if type(thick) in [int,float] :self.thick=thick
        self._set_kinematic()
        if self.solved:self._set_intensities()

    def solve(self,Smax=0.02,Nmax=None,K=None,u=None,keV=None,thick=None,
        opts='sv',Vopt0=False,v=True,name=''):
        ''' diagonalize the Blochwave matrix
        - Smax    : float- maximum excitation error to be included
        - Nmax    : int - max order of reflections/resolution(see update_Nmax)
        - K,u,keV : reciprocal space beam vector(see update beam)
        - opts  : 'str - s'(save) '0'(Vopt0) 'v'(verbose) 'H'(show H)
        - Vopt0 : bool - set Vg0=0
        - v     : bool - verbose
        '''
        self.update_Nmax(Nmax)
        self.set_beam(K,u,keV)
        self.set_name(name,self.path)
        self._set_excitation_errors(Smax)
        self._solve_Bloch(opts,Vopt0,v)
        self._set_Vg()
        self.set_thickness(thick)
        self._set_kinematic()
        if 's' in opts:self.save()

    ################################################################################
    #### private
    ################################################################################
    def _solve_Bloch(self,opts='0',Vopt0=True,v=0):
        ''' Diagonalize the Hamiltonian
        Ug is a (2*Nmax+1)^3 tensor :
        # Ug[l] = [U(-N,-N,l) .. U(-N,0,l) .. U(-N,N,l)
        #          U( 0,-N,l) .. U( 0,0,l) .. U( 0,N,l)
        #          U(-N,-N,l) .. U( N,0,l) .. U( N,N,l)]
        '''
        Vopt0 = Vopt0 or '0' in opts
        v = v or 'v' in opts

        hkl = self.df_G[['h','k','l']].values
        Sg  = self.df_G.Sw.values
        Ug = self.Fhkl/self.crys.volume #/3

        # Ug[U0_idx] = U0
        # and Ug(iG,jG) are obtained from Ug[h,k,l] where h,k,l = hlk_iG-hkl_jG
        U0_idx = [2*self.Nmax]*3
        if Vopt0 : Ug[tuple(U0_idx)] = 0   #setting average potential to 0

        if v:print(colors.blue+'...assembling %dx%d matrix...' %((Sg.shape[0],)*2)+colors.black)
        H = np.diag(Sg+0J)
        for iG,hkl_G in enumerate(hkl) :
            U_iG = np.array([Ug[tuple(hkl_J+U0_idx)] for hkl_J in hkl_G-hkl]) #;print(V_iG.shape)
            # print('G : ',hkl_G)
            # print('U_G:',Ug[tuple(U0_idx+hkl_G)])
            # print('idx iG:',[tuple(hkl_J+U0_idx) for hkl_J in hkl_G-hkl])
            # print(U_iG)
            H[iG,:] += U_iG/(2*self.k0)  #off diagonal terms as potential

        self.H = H
        if 'H' in opts:self.show_H()

        if v:print(colors.blue+'...diagonalization...'+colors.black)
        self.gammaj,self.CjG = np.linalg.eigh(H) #;print(red+'Ek',lk,black);print(wk)
        self.invCjG = np.linalg.inv(self.CjG)
        self.solved = True

    def _set_excitation_errors(self,Smax=0.02):
        ''' get excitation errors for Sg<Smax
        - Smax : maximum excitation error to be included
        '''
        K,K0 = self.K,self.k0
        (h,k,l),(qx,qy,qz) = self.lattice

        Kx,Ky,Kz = K
        Sw = np.abs(np.sqrt((Kx-qx)**2+(Ky-qy)**2+(Kz-qz)**2) - K0)
        if Smax:
            idx = Sw<Smax
            h,k,l = np.array([h[idx],k[idx],l[idx]],dtype=int)
            qx,qy,qz,Sw = qx[idx],qy[idx],qz[idx],Sw[idx]
        d = dict(zip(['h','k','l','qx','qy','qz','Sw'],[h,k,l,qx,qy,qz,Sw]))

        self.Smax = Smax
        self.nbeams = Sw.size
        self.df_G = pd.DataFrame.from_dict(d)
        self.solved = False

    def _set_Vg(self):
        Vg     = self.Fhkl.copy()/self.crys.volume
        V0_idx = np.array([2*self.Nmax]*3)
        hkl    = self.df_G[['h','k','l']].values
        Vg[tuple(V0_idx)] = 0
        Vg_G       = np.array([ Vg[tuple(hkl_G+V0_idx)] for hkl_G in hkl])
        px,py,e0x  = mut.project_beams(K=self.K,qxyz=self.get_G(),e0=[1,0,0],v=1)
        self.e0x = e0x
        self.df_G['px'] = px
        self.df_G['py'] = py
        self.df_G['Vg'] = Vg_G
        self.df_G['L']  = np.ones(Vg_G.shape)
        # self.df_G.loc[(self.df_G.h==0) & (self.df_G.k==0) & (self.df_G.l)==0]['Vg'] = 0
        self._set_zones()

    def _set_zones(self):
        hkl     = self.df_G[['h','k','l']].values
        # Khkls   = np.round(hkl.dot(self.Kuvw/np.linalg.norm(self.Kuvw))*100)/100
        Khkls    = np.round(hkl.dot(self.Kuvw0)*100)/100 #integer only in zone axis orientation
        Khkl,ar = np.unique(Khkls,return_inverse=True) #; print(Khkls);print(zones);print(ar)
        zones = np.argsort(Khkl)
        self.df_G['zone'] = zones[ar]

    def _set_intensities(self):
        '''get beam intensities at thickness'''
        id0 = self.is_hkl([0,0,0],v=0)
        gammaj,CjG = self.gammaj,self.CjG
        S = CjG.dot(np.diag(np.exp(2J*np.pi*gammaj*self.thick))).dot(self.invCjG)
        # S = S[:,id0]
        S = S[id0,:]
        self.df_G['S'] = S
        self.df_G['I'] = np.abs(S)**2

    def _set_kinematic(self):
        Sw,Ug = self.df_G[['Sw','Vg']].values.T
        t,sig = self.thick, self.sig

        #[Ug]=[A-2], [k0]=[A^-1], [t]=[A], [Fhkl]=[fe]=[A]
        Sg = np.pi/self.k0*Ug*t*np.sinc(Sw*t)
        self.df_G['Sg'] = Sg
        self.df_G['Ig'] = np.abs(Sg)**2

    def set_beams_vs_thickness(self,thicks):
        ''' get Scattering matrix as function of thickness for all beams
        - thicks : tuple(ti,tf,step) or list or np.ndarray thicknesses
        '''
        self._set_thicks(thicks)
        id0 = self.is_hkl([0,0,0],v=0)
        St = np.zeros((self.df_G.shape[0],self.z.size),dtype=complex)
        gammaj,CjG,invCjG = self.gammaj,self.CjG,self.invCjG
        for iT,thick in enumerate(self.z):
            # S=CjG.T.dot(np.diag(np.exp(2J*np.pi*gammaj*T))).dot(CjG)
            S = CjG.dot(np.diag(np.exp(2J*np.pi*gammaj*thick))).dot(invCjG)
            # St[:,iT]=CjG.dot(np.diag(np.exp(2J*np.pi*gammaj*T))).dot(CjG.T)[0,:]
            St[:,iT] = S[:,id0]
        self.Sz = St
        self.Iz = np.abs(self.Sz)**2

    def _set_thicks(self,thicks):
        if isinstance(thicks,tuple):thicks = np.linspace(*thicks)
        elif isinstance(thicks,float) or isinstance(thicks,int) : thicks=[thicks]
        self.z = np.array(thicks)

    ################################################################################
    #### getter
    ################################################################################
    def get_intensities(self):return self.df_G.I
    def get_hkl(self):return self.df_G[['h','k','l']].values
    def get_kin(self):return self.df_G[['h','k','l','Sw','Vg','Sg','Ig']]
    def get_zones(self):return self.df_G[['h','k','l','zone']].values
    def get_G(self):return self.df_G[['qx','qy','qz']].values
    def get_Istrong(self,m={'I':1000,'Ig':1000,'Vg':10},out=0,Icols=['I']):
        cond = self.df_G.L<0
        Icols = [Icol for Icol in Icols if Icol in self.df_G.columns and Icol in m.keys()]
        for Icol in Icols :
            Imax = abs(self.df_G[Icol].max())
            cond |= (abs(self.df_G[Icol])>Imax/m[Icol])
        Istrong = self.df_G.loc[cond]
        if out:return Istrong.index.values
        else:
            if any(Istrong):print(Istrong[['h','k','l','Sw']+Icols])

    def get_Sw(self,Smax=1):
        Smax = min(Smax,self.Smax)
        print(self.df_G[['h','k','l','Sw']].loc[self.df_G.Sw<=Smax])
    def is_hkl(self,Ghkl,v=1):
        Gidx = np.where(np.linalg.norm(self.get_hkl() - Ghkl,axis=1)==0)[0]
        if v:print(self.df_G.iloc[Gidx])
        return Gidx[0]

    ################################################################################
    #### display
    ################################################################################
    def show_beams(self,F='I',fopts='m',opts='xN',mag=500,cutoff=0,cmap='Greens',**kwargs):
        '''Display beam values
        - F : 'I'(intensity),'S'(scattered beams),'Vg'(potential)
        - opts  : 'x'(include projection of x axis), 'N'(normalize)
        - fopts : see get_fz
        '''
        fz,fz_str = get_fz(fopts)
        F_str = {'L':'Lattice','Vg':'Potential $V_{g}$',
            'S' :'Scattered beams Bloch $S_{g}$',
            'I' :'Intensity Bloch$I_{g}$',
            'Sg':'Scattered beams Kinematic $S_{g,kin}$',
            'Ig':'Intensity kinematic $I_{g,kin}$',
            'Sw':'Excitation error $\zeta_{g}$'}[F]

        tle = '%s, %s, thickness=%d$\AA$'  %(F_str,fz_str,self.thick)
        px,py = self.df_G[['px','py']].values.T
        Fvals = fz(self.df_G[F])
        if 'N' in opts:Fvals/=Fvals.max()
        # mag /= Fvals.max()
        scat  = [px,py,Fvals*mag/Fvals.max(),Fvals]
        plts = [[0,0,'b+','']]
        if 'x' in opts:plts+=[ [[0,self.e0x],[0,0],'r','']]
        if not cutoff:cutoff = Fvals.max()
        # print(Fvals.max())
        fig,ax = dsp.stddisp(plts,lw=2,scat=scat,caxis=[0,cutoff],cmap=cmap,cs='S',title=tle,**kwargs)

    def show_beams_vs_thickness(self,thicks=None,strong=[],**kwargs):
        if thicks:self.set_beams_vs_thickness(thicks)
        hkl = self.get_hkl().copy()
        Iz  = self.Iz.copy()
        if any(strong):
            Istrong = self.get_Istrong(out=1,Icols=strong) #;print(Istrong)
            Iz  = Iz[Istrong]
            hkl = hkl[Istrong]
        hkl = [str(tuple(h)) for h in hkl]
        beams=[hkl,self.z,None,None,Iz]
        # beams=[hkl,self.z,np.real(self.Sz),np.imag(self.Sz),self.Iz]
        return pp.plot_beam_thickness(beams,**kwargs)

    def show_Fhkl(self,s=None,opts='m',h3D=0,**kwargs):
        '''Displays structure factor over grid
        - opts : see get_fz
        - s : slice or str('k=0' => Fhkl(k=0)) or int('l==<int>')
        '''
        fz,fz_str = get_fz(opts)
        s,s_str = self._get_slice(s)
        tle = 'Structure factor($\AA$), showing %s in %s'  %(fz_str,s_str)
        h,k,l = self.hklF
        Fhkl  = self.Fhkl #.copy()
        if isinstance(s,tuple):
            Fhkl   = Fhkl[s]
            nx,ny  = np.array((np.array(Fhkl.shape)-1)/2,dtype=int)
            i,j    = np.meshgrid(np.arange(-nx,nx+1),np.arange(-ny,ny+1))
            fig,ax = dsp.stddisp(scat=[i,j,fz(Fhkl)],title=tle,**kwargs)
        else:
            fig,ax = dsp.stddisp(scat=[h,k,l,fz(self.Fhkl)],title=tle,rc='3d',**kwargs)
            if h3D:h3d.handler_3d(fig,persp=False)

    def show_H(self,**kwargs):
        dsp.stddisp(im=[np.abs(self.H)],title='abs(H)', pOpt='im')

    ################################################################################
    #### misc
    ################################################################################
    def _get_slice(self,s):
        tle = ''
        if isinstance(s,tuple) or isinstance(s,int) or isinstance(s,str):
            if isinstance(s,int):s = 'l=%d' %s
            if isinstance(s,str):
                i,n = s.split('=')
                tle = 'plane %s, ' %s
                n   = 2*self.Nmax+int(n)
                if   i=='h':s = np.s_[n,:,:]
                elif i=='k':s = np.s_[:,n,:]
                elif i=='l':s = np.s_[:,:,n]
        return s,tle
    def save(self,file=None,v=1):
        '''save this object'''
        if not file:file=os.path.join(self.path,self.name+'.pkl')
        with open(file,'wb') as out :
            pickle5.dump(self, out, pickle5.HIGHEST_PROTOCOL)
        if v:print(colors.green+"object saved\n"+colors.yellow+file+colors.black)



################################################################################
#### functions
################################################################################
def load_Bloch(path='',tag='',file='',):
    '''load a saved Bloch object
    filename : pickle file (.pkl)  '''
    files = [f for f in glob.glob(os.path.join(path,'*.pkl')) if tag in f]
    if tag:
        if len(files)==1:
            file = files[0]
    if file:
        with open(file,'rb') as f : obj = pickle5.load(f)
        print(colors.green+'loaded:' +colors.yellow+file+colors.black);
        return obj
    else:
        if len(files):
            print(colors.red+'mulitple simus available with tag %s:' %tag);
            print(colors.yellow,[os.path.basename(f) for f in files],colors.black)
        else:
            print(colors.red+'no simus available with in "%s" with tag %s' %(path,tag))

def get_fz(opts):
    '''mapping for function to apply
    - opts : 'r'(real) 'i'(imag) 'a'(angle) 'm'(mag) 'l'(log10(|Fhkl|+1)) '2'(mag^2) 'L'(logM)
    '''
    if not opts:opts='m'
    keys   =['r','i','a','m','l','2','L']
    fs     = [np.real,np.imag,np.angle,np.abs,logF,abs2,logM]
    fs_str = ['real part','imag part','phase','magnitude','$\log_{10}(|F|+1)$','$|F|^2$','$-\log_{10}$']
    fz     = dict(zip(keys,fs))[opts]
    fz_str = dict(zip(keys,fs_str))[opts]
    return fz,fz_str

abs2 = lambda F:np.abs(F)**2
logF = lambda F:np.log10(np.abs(F)+1)
logM = lambda F:-np.log10(np.maximum(F,1e-5))



import importlib as imp
from scipy.integrate import nquad,trapz,quad
from scipy.interpolate import interp1d
from utils import*                  ;imp.reload(dsp)
import wallpp.plane_group as pg     ;imp.reload(pg)
import multislice.multi_2D as MS2D  ;imp.reload(MS2D)
plt.close('all')
path = '../multislice/docs_fig/'


## structure
pptype,a,b,angle = 'p1',50,5,90
pattern = np.array([[25,2,1]])   #pattern
ndeg    = 2**11                  #sampling per unit cell
p1      = pg.Wallpaper(pptype,a,b,angle,pattern,ndeg=ndeg)
pattern = p1.get_potential_grid()
eps=1      #transmission function exp(j*eps*sig*Vz)


def manual_convolution(x,T,k0,angle,Dz):
    dx,b0 = angle*Dz,x.max()

    #real space propagator
    xx = np.linspace(-dx,dx,10000)
    P  = -1J*np.exp(1J*np.pi/4)*k0/Dz*np.exp(1J*np.pi*k0/Dz*xx**2)
    cc,ss = P.real,P.imag
    plts0 = [[xx,cc,'b','$P_r$'],[xx,ss,'r','$P_i$']]
    dsp.stddisp(plts0,labs=['$x$','Fresnel Propagator'],lw=2,xylims=['x',-2,2],name=path+'fresnelP.svg',opt='ps')

    # transmission
    x0,Tr,Ti = np.hstack([-dx,x,b0+dx]),np.hstack([1,T.real,1]),np.hstack([0,T.imag,0])
    plts = [[x0,Tr,'b','$T_r$'],[x0,Ti,'r','$T_i$']]
    dsp.stddisp(plts,labs=['$x$','$Transmission$'],lw=2,name=path+'fresnelT.svg',opt='ps')
    Tr,Ti = interp1d(x0,Tr),interp1d(x0,Ti)

    ### integrands
    TrGr = lambda x,xp:Tr(xp-x)*cc
    TrGi = lambda x,xp:Tr(xp-x)*ss
    TiGr = lambda x,xp:Ti(xp-x)*cc
    TiGi = lambda x,xp:Ti(xp-x)*ss
    psi_fr = np.zeros(T.shape,dtype=complex)
    rr,ii = np.zeros(T.shape),np.zeros(T.shape)
    ri,ir = np.zeros(T.shape),np.zeros(T.shape)
    for j in range(0,x.size,1): #loop over observation points
        rr[j] = trapz(TrGr(xx,x[j]),xx)
        ri[j] = trapz(TrGi(xx,x[j]),xx)
        ir[j] = trapz(TiGr(xx,x[j]),xx)
        ii[j] = trapz(TiGi(xx,x[j]),xx)
        psi_fr[j]=rr[j]-ii[j]+1J*(ri[j]+ir[j])
    # plts0  = [[mp0.x,rr,'b'  ,'rr']      ,[mp0.x,ri,'r'  ,'ri']]
    # plts0 += [[mp0.x,ii,'b--','ii']      ,[mp0.x,ir,'r--','ir']]
    # dsp.stddisp(plts0,labs=['$x$','$\Psi(x)$'],lw=2,name=path+'fresnelTG.svg',opt='p')
    return psi_fr

#multislice 1 slice
print('...ms...')
mp0 = MS2D.Multi2D(pattern,a,b,keV=200,
        Nx=1,dz=b,nz=1,ppopt='',#XQZTP
        iZs=1,iZv=1,sg=-1,eps=eps,copt=1)
psi_cv = mp0.Psi_x

#manual convolution
print('...convolution...')
angle = 10*mp0.q.max()/mp0.k0/2
psi_fr = manual_convolution(x=mp0.x,T=mp0.T[0], k0=mp0.k0,angle=angle,Dz=b)


####real space
frr = psi_fr.real/psi_fr.real[0]
fri = psi_fr.imag/psi_fr.real[0]
ftr = psi_cv.real/psi_cv.real[0]
fti = psi_cv.imag/psi_cv.real[0]

plts   = [[mp0.x,frr,'b','frr']     ,[mp0.x,fri,'r','fri']]
plts  += [[mp0.x,ftr,'b--','Re(MS)'],[mp0.x,fti,'r--','Im(MS)']]
dsp.stddisp(plts,labs=['$x$','$\Psi(x)$'],lw=2,xylims=['x',20,30],name=path+'fresnelX.svg',opt='ps')



import importlib as imp
from utils import*
import multislice.pymultislice as pyms;imp.reload(pyms)
import multislice.multislice as ms    ;imp.reload(ms)
import multislice.mupy_utils as mut   ;imp.reload(mut)
import multislice.postprocess as pp   ;imp.reload(pp)
import multislice.multi_3D as MS3D    ;imp.reload(MS3D)
# from crystals import Crystal
plt.close('all')
figpath='../../multislice/docs_fig/multi3D/'
datpath='../../multislice/dat/Silicon/Si110/'
Si110 = datpath+'Si110.xyz'

# opts = 'pTl' #p(py)l(load) T(temsim)L(Load)
ndz=2
nxy=2**10
pattern,(ax,by,cz) = mut.import_xyz(Si110)
pattern = pattern[:,:4]
dz = cz/ndz

def run_py3D(load_opt=0,opts='stql',Nz=None,nz=None,v=1):
    mp_py = mut.multi3D(load_opt=load_opt,name=datpath+'Si110',
        pattern=pattern,ax=ax,by=by,cz=cz,
        keV=200,Nxy=[1,1],nxy=nxy,dz=dz,Nz=Nz,nz=nz,
        TDS=False,
        iZs=1,iZv=1,opts=opts,v=v,
        hk=3,hkopt='r',
        eps=1,copt=1,temsim=True,
        ppopt='')
    # mp_py.Bz_show(cm='jet')
    return mp_py


def run_temsim(load_opt=0,Nz=1):
    if load_opt:
        mp_ms = pp.load_multi_obj(datpath+'Si110_autoslic.pkl')
    else:
        mp_ms = ms.Multislice(datpath,mulslice=False,data=Si110,
            keV=200,slice_thick=dz,NxNy=nxy,repeat=[1,1,Nz],Nhk=5,
            v=1,opt='rs',ppopt='w',fopt='f',i_slice=1)
    # mp_ms.pattern(Iopt='Icns',cmap='gray',imOpt='ch',xylims=[0,12,0,12],
    #     name='docs_fig/Si110_autoslic_pattern.png',axPos='T',setPos=1)
    # mp_ms.beam_vs_thickness(bOpt='fa',tol=1e-4)
    return mp_ms

def compare_transmission(iz,plot=0,**kwargs):
    mp_py = pyms.load(datpath+'Si110_3D.pkl')
    Tpy = mp_py._get_transmission_function(iz,v=2,copt=1,save=0,load=1)
    Tms = np.loadtxt(datpath+'translayer.%s' %str(iz).zfill(3) )
    Tms = Tms[:,::2]+1J*Tms[:,1::2]
    deltaT = Tms-Tpy
    print('max diff re=%.2E, im=%.2E' %(abs(np.real(deltaT)).max(),abs(np.imag(deltaT)).max()) )
    if 'e' in plot:
        if 'r'in plot:dsp.stddisp(im=[deltaT.real],pOpt='im',title='re error(T) iz=%d' %iz,**kwargs)
        if 'i'in plot:dsp.stddisp(im=[deltaT.imag],pOpt='im',title='im error(T) iz=%d' %iz,**kwargs)
    if 't' in plot:
        if 'r'in plot:dsp.stddisp(im=[Tms.real],pOpt='im',title='TEMSIM Re T iz=%d' %iz,**kwargs)
        if 'r'in plot:dsp.stddisp(im=[Tpy.real],pOpt='im',title='MULT3D Re T iz=%d' %iz,**kwargs)
        if 'i'in plot:dsp.stddisp(im=[Tms.imag],pOpt='im',title='TEMSIM Im T iz=%d' %iz,**kwargs)
        if 'i'in plot:dsp.stddisp(im=[Tpy.imag],pOpt='im',title='MULT3D Im T iz=%d' %iz,**kwargs)
    return Tpy,Tms

def compare_propagator(opts='qri'):
    P = np.loadtxt(datpath+'propagator.txt')
    nx = int(P.shape[0]/2)
    Px,Py = P[:nx,0]+1J*P[:nx,1] , P[nx:,0]+1J*P[nx:,1]
    if opts:
        P = np.outer(Px,Py)
        x = np.arange(nx)
        plts = [[x,Px.real,'b','Re Px'],[x,Px.imag,'r','Im Px']]
        plts+= [[x,Py.real,'c','Re Py'],[x,Py.imag,'m','Im Py']]
        if 'q' in opts:dsp.stddisp(plts,labs=['q','Px,Py'])
        if 'r' in opts:dsp.stddisp(im=[P.real.T],pOpt='im',title='Re P temsim')
        if 'i' in opts:dsp.stddisp(im=[P.imag.T],pOpt='im',title='Im P temsim')
    return Px,Py

def compare_patterns(iz,plot=0,**kwargs):
    mp_py = pyms.load(datpath+'Si110_3D.pkl')
    I_py = mp_py.pattern(iz)
    mp_ms = pyms.load(datpath+'Si110_3D.pkl')
    # I_ms = mp_py.pattern(iz)
    qx,qy,I_ms = mp_ms.pattern(iz,file=datpath+'',out=1)

mp_py = run_py3D(load_opt=1,opts='stq',Nz=5,v=1)
mp_ms = run_temsim(load_opt=1,Nz=5)
# Px_ms,Py_ms = compare_propagator(opts='r')
# Px_py,Py_py = mp_py.Pq_show(opts='r')
# Tpy,Tms = compare_transmission(2,plot='tr')
# mp_py.Tz_show(0,'i')

hk = [hk0 for hk0 in mp_py.hk if hk0 in mp_ms.hk]
fig,ax = mp_py.Bz_show(           iBs=hk,cm='jet', linespec='-' ,opt='')#title='py  3D')
mp_ms.beam_vs_thickness(bOpt='fa',iBs=hk,cm='jet', linespec='--',ax=ax,legOpt=0)#title='temsim')



# from utils.handler3D import handler_3d
import importlib as imp
from utils import*                          ; imp.reload(dsp)
import multislice.multislice as mupy        ; imp.reload(mupy)
import multislice.postprocess as pp         ; imp.reload(pp)
import multislice.mupy_utils as mut         ; imp.reload(mut)
import multislice.rotating_crystal as rcc   ; imp.reload(rcc)
from crystals import Crystal
plt.close('all')
path = '../dat/ireloh/'
figpath = '../docs_fig/ireloh/'
cif_file = path+'ireloh.cif'
opts = 'FT'  # F(structure factor) E(Ewald) T(tilts) B(Beams)
hklM = 20

rotx = lambda a:np.array([[1,0,0],[0,np.cos(a),np.sin(a)],[0,-np.sin(a),np.cos(a)]])
rotz = lambda a:np.array([[np.cos(a),np.sin(a),0],[-np.sin(a),np.cos(a),0],[0,0,1]])

imV = mut.Image_viewer('/home/tarik/Documents/data/ireloh/IRELOH_ED_Dataset_1/n14_a004_0521.cbf')
fig,ax = imV.show_image(stack=5,caxis=[0,50],cmap='binary',opt='p')

# mut.show_image('/home/tarik/Documents/data/ireloh/IRELOH_ED_Dataset_1/n14_a004_0484.cbf')
# exit()
# cbf_v = mut.Viewer_cbf(exp_path='/home/tarik/Documents/data/ireloh/IRELOH_ED_Dataset_1/',figpath=figpath)

rsa = np.array([
    [  5.75197579,   1.08642157,   5.77468802],
    [ -1.09493538,  -9.45325930,   2.86911892],
    [ 12.49046736,  -4.94063233, -11.5118351 ],])
rsa_ = mut.get_reciprocal(rsa)
# rsa_ :  np.array([
#  [ 0.09721788 ,  0.05915872,  0.06860005],
#  [-0.01449509 , -0.07170057,  0.07450839],
#  [ 0.04263771 , -0.03194969, -0.0268509 ],])
lat_rec = np.linalg.norm(rsa_,axis=1)

# get the indexing matrix A-1
A = np.array([
[0.08540147986530082, -0.010093029325705745, 0.03984388435141725],
[0.016954218646467147, -0.09572669491816205, -0.015506584465560233],
[0.08490755747184861, 0.029266310687469856, -0.036979330900490597],
])
Ainv = np.linalg.inv(A)

#identified reflections
# xh0s = np.array([[-0.05,0.63],[-0.25,0.14],[-0.48,0.16,0],[-0.16,-0.37,0],[-0.49,-0.18,0]])
xh0s = np.array([[-0.49,-0.18],[-0.16,-0.37],[-0.558,0.329],[0.311,0.257],[-0.25,0.14]])
xh0s = np.hstack([xh0s,np.zeros((xh0s.shape[0],1))])*0.3
x0,y0 = 0.07*0.3,-0.07*0.3 #270,270,
xh = np.array(xh0s)-np.array([x0,y0,0])

phi = np.deg2rad(-29.014)
xh_r = rotx(-phi).dot(xh.T).T
# h = rsa_.dot(xh)/lat_rec**2;#print(h)
hs = Ainv.dot(xh_r.T).T
hstr = [ ','.join(['%.1f' %i for i in h ]) for h in hs]


#plot reflections and axi
txts = [[x[0]+x0,x[1]+y0,l,c] for x,l,c in zip(rsa_,['$a^{*}$','$b^{*}$','$c^{*}$'],['r']*3)]
arrs = [[x0,y0,x[0],x[1],c] for x,c in zip(rsa_,['r']*3)]
txts += [[xh0[0],xh0[1],'(%s)' %h_str,'k'] for h_str,xh0 in zip(hstr,xh0s)]
#resolution rings
t = np.linspace(0,2*np.pi,500)
ct,st = np.cos(t),np.sin(t)
plts = [[R*ct,R*st,'r',''] for R in 1/np.array([1.32,0.93,0.76,0.66])]
#display
imV = mut.Image_viewer('/home/tarik/Documents/data/ireloh/IRELOH_ED_Dataset_1/n14_a004_0521.cbf')
# fig,ax = imV.show_image(stack=5,caxis=[0,50],cmap='binary',opt='')
# dsp.stddisp(plts,ax=ax,fig=fig,texts=txts,arrows=arrs,labs=['$q_x$','$q_y$'],
#     opt='p',xylims=0.2)

xy_px = np.array(imV.s2px(xh0s)).T
print('xy_px:',xy_px)
print('xy_mm :',xy_px*imV.pxy*1000)


def show_setup(name,gopts=['xy','yz','xz'],**kwargs):
    xyz_name = dsp.basename(name)
    for opts in gopts:
        figname=figpath+xyz_name.replace('.xyz','')+opts+'.png'
        mut.show_grid(name,opts,name=figname,**kwargs)

    # dials.frame_orientations integrated.expt

    # n= np.array([-1249.0467359091645, 494.06323332809154, 1151.1835101484317])/200
    # n = 10*np.array([-1.51089556, -0.51374498, 0.65074564])
    # array([-0.08540403, -0.02904403,  0.0367669 ])
    angles = np.deg2rad([134.61, 106.78, 49.40])
    crys = Crystal.from_cif(cif_file)
    lat_vec = np.array(crys.lattice_vectors)
    lat_params = np.array(crys.lattice_parameters[:3])


    n= np.array([-0.08540851, -0.02904118, 0.03678561])*lat_params
    ## original structure
    # name = path+'ireloh111.xyz'
    # rcc.import_cif(cif_file,xyz=name)
    #
    # fig,ax = mut.show_grid3(name,opt='',ms=50)
    # rcc.show_unit_cell(ax,a=0.2,c='b',lw=2,lat_params=crys.lattice_parameters[:3])
    # rcc.show_trihedron(ax,uvw=np.array([n]) ,x0=[0,0,0],cs=['k--'],clabs=['beam'],lw=3,rc=0.1,h=0.2,opt='')
    # rcc.show_trihedron(ax,uvw=lat_vec,x0=[0,0,0],cs=['r--','g--','b--'],clabs=['$a$','$b$','$c$'],lw=3,rc=0.1,h=0.2,
    #     labs=['$x$','$y$','$z$'],xylims=[-10,15,-10,15,-5,20],view=[10,-20],name=figpath+'1_484_xyz.png',opt='p')
    #
    # ## rotated structure
    ez=[0,0,-1]

    uvw  = rcc.orient_crystal(lat_vec,n_u=n/np.linalg.norm(n),ez=ez)
    n_rot= rcc.orient_crystal(n      ,n_u=n/np.linalg.norm(n),ez=ez)

    np.rad2deg(np.arccos(uvw.dot([0,0,-1])/lat_params))


    a = 0
    # a = np.deg2rad(225)

    uvw = np.array([rotz(a).dot(u) for u in uvw])
    # n_rot = rotz(a).dot(n_rot)

    mut.import_cif(cif_file,xyz=name,n=n)
    #
    # fig,ax = mut.show_grid3(name,opt='',ms=50)
    # rcc.show_unit_cell(ax,n=n,a=0.2,c='b',lw=2,lat_params=crys.lattice_parameters[:3])
    # rcc.show_trihedron(ax,uvw=np.array([n_rot]) ,x0=[0,0,0],cs=['k'],clabs=['beam'],lw=3,rc=0.1,h=0.2,opt='')
    # rcc.show_trihedron(ax,uvw=uvw    ,x0=[0,0,0],cs=['r','g','b'],clabs=['$a$','$b$','$c$'],lw=3,rc=0.1,h=0.2,
    #     labs=['$x$','$y$','$z$'],xylims=[-10,20,-10,20,-10,20],view=[20,-100],name=figpath+'1_484_rot.png',opt='p')


    # fig,ax = mut.show_grid3(name,opt='',ms=50)
    # name = path+'ireloh111_rotated484.xyz'
    # rcc.show_unit_cell(ax,n=n,a=0.2,c='b',lw=2,lat_params=crys.lattice_parameters[:3])
    # rcc.show_trihedron(ax,uvw=np.array([n_rot]) ,x0=[0,0,0],cs=['k'],clabs=['beam'],lw=3,rc=0.1,h=0.2,opt='')
    # rcc.show_trihedron(ax,uvw=uvw    ,x0=[0,0,0],cs=['r','g','b'],clabs=['$a$','$b$','$c$'],lw=3,rc=0.1,h=0.2,
    #     labs=['$x$','$y$','$z$'],xylims=[-10,20,-10,20,-10,20],view=[20,10],name=figpath+'1_484_dav.png',opt='p')

    rsa = np.array([
        [  5.75197579,   1.08642157,   5.77468802],
        [ -1.09493538,  -9.45325930,   2.86911892],
        [ 12.49046736,  -4.94063233, -11.5118351 ],])
    rsa_ = mut.get_reciprocal(rsa)

    # name = path+'ireloh111_SRFU.xyz'
    a,b,c = crys.lattice_parameters[:3]
    fig,ax=dsp.stddisp(rc='3d')
    # fig,ax = mut.show_grid3(name,opt='',ms=50)
    # rcc.show_unit_cell(ax,n=n,ez=ez,a=0.2,c='b',lw=2,lat_params=[a,b,c])
    # rcc.show_unit_cell(ax,a=0.05,c='b',lw=2,lat_params=[a,b,c],ls='--')
    rcc.show_trihedron(ax,uvw=np.array([10*n,10*n_rot,[10,0,0]]) ,x0=[0,0,0],cs=['k-.','k--','k-'],clabs=['beam','$b_z$','phi'],lw=3,rc=0.1,h=0.2)
    # rcc.show_trihedron(ax,uvw=np.array() ,x0=[0,0,0],cs=['k'],clabs=['beam'],lw=3,rc=0.1,h=0.2,opt='')
    # rcc.show_trihedron(ax,uvw=rsa    ,x0=[0,0,0],cs=['r-','g-','b-']   ,clabs=['$a$','$b$','$c$'],lw=3,rc=0.1,h=0.2)
    rcc.show_trihedron(ax,uvw=rsa_   ,x0=[0,0,0],cs=['r--','g--','b--'],clabs=['$a^{*}$','$b^{*}$','$c^{*}$'],lw=3,rc=0.1,h=0.2)
    # rcc.show_trihedron(ax,uvw=uvw1   ,x0=[0,0,0],cs=['r','g','b'],clabs=['$a1$','$b1$','$c1$'],lw=3,rc=0.1,h=0.2)
    # rcc.show_trihedron(ax,uvw=uvw    ,x0=[0,0,0],cs=['r','g','b'],clabs=['$a$','$b$','$c$'],lw=3,rc=0.1,h=0.2,

    dsp.stddisp(ax=ax,fig=fig,labs=['$x$','$y$','$z$'],xylims=np.array([-10,20,-10,20,-10,20]),view=[20,10],name=figpath+'1_484_dav.png',opt='p')

    # hdl = handler_3d(fig,persp=False)

# name = path+'ireloh001.xyz'
# name = path+'ireloh484.xyz'
# name = path+'IRELOH001_test.xyz'
# name = path+'IRELOH001.xyz'
# name = path+'ireloh_rotated484.xyz'
name = path+'ireloh111_rotated484.xyz'
# name = path+'ireloh_rotated900.xyz'
# name = path+'ireloh111_rotated484_TD.xyz'

# show_setup(name,gopts=[],opt='sc')


def xyz_gif(rpath,dpath):
    rpath = '/home/tarik/Documents/git/ccp4/reports/ireloh/xyz/'
    dpath='/home/tarik/Documents/git/ccp4/src/david_scripts/ED_processing/IRELOH_ED_Dataset_1-dials/'
    for i in range(500,1351,50) :
        print(i)
        name = dpath+'ireloh_rotated%d.xyz' %i
        mut.show_grid(name,'xy',name=rpath+'xy/1_xy%s.png' %(str(i).zfill(4)),opt='sc',xylims=[100,450,100,450])
        mut.show_grid(name,'yz',name=rpath+'yz/1_yz%s.png' %(str(i).zfill(4)),opt='sc',xylims=[100,500,0,310])
        mut.show_grid(name,'xz',name=rpath+'xz/1_xz%s.png' %(str(i).zfill(4)),opt='sc',xylims=[100,450,0,310])


# mut.show_grid(name,'xz')#,xylims=[100,450,100,450],name=figpath+'ireloh_rotated484_xy.png',opt='sc')



def run_simu(name):
    pad,rep = 1,[1,1,1]
    # mut.import_cif(cif_file,name,n=[0,0,1],rep=rep,pad=pad)#dopt='s')

    # h,k = np.meshgrid(np.arange(-6,7)*(rep[0]+2*int(pad)),np.arange(-6,7)*(rep[1]+2*int(pad)))
    h,k = np.meshgrid(np.arange(-30,31),np.arange(-30,31))
    hk=[(h0,k0) for h0,k0 in zip(h.flatten(),k.flatten())];#print(hk)

    multi = mupy.Multislice(path,data=name,tail='900',
        mulslice=False,keV=200,#tilt=[tx*np.pi/180/1000,0],
        NxNy=2**12,repeat=[1,1,1],slice_thick=1,hk=hk,#Nhk=5,
        #TDS=True,T=300,n_TDS=15,
        opt='srp',fopt='f',v='nctr',ppopt='wuPBf',#nctrdDR',
        ssh='badb',#hostpath=hostpath
        )
# run_simu(name)


def postpp(multi):
    fig,ax = multi.pattern(Iopt='Incs',out=False,Nmax=256,#tol=1e-3,
        # gs=1.3,caxis=[-6.2,0],
        caxis=[0,0.001],
        rings=[0.25,0.5],lw=2,
        imOpt='cv',axPos='V',cmap='gray',
        xylims=0.4*np.array([-1,1,-1,1]),name=figpath+'900_I.png',opt='')

    qx,qy,I = multi.pattern(Iopt='Incs',out=True,Nmax=256,opt='c')

    Nmax = 256
    q = np.sqrt((qx-qx.min())**2+(qy-qy.min())**2)

    nx,ny,nd = 2,2,4
    ss = [np.s_[Nmax+int(i*Nmax/nx),:] for i in range(-nx+1,nx)]
    ss+= [np.s_[:,Nmax+int(i*Nmax/ny)] for i in range(-ny+1,ny)]
    ss+=[np.s_[range(2*Nmax-1,-1,-1),range(2*Nmax)],np.s_[range(2*Nmax-1,-1,-1),range(2*Nmax-1,-1,-1)]]
    sl = ['$q_y=%d$' %i for i in range(-nx+1,nx)]+['$q_x=%d$' %i for i in range(-ny+1,ny)]+['$q_x=-q_y$','$q_x=q_y$']

    sc=dsp.getCs('Spectral',len(ss))
    plts=[[qx[s],qy[s],c,l] for s,l,c in zip(ss,sl,sc)]
    dsp.stddisp(plts,fig=fig,ax=ax,lw=2)

    dsp.stddisp([[q[s],I[s],c,l] for s,l,c in zip(ss,sl,sc)],lw=2,xylims=['y',0,0.001])

    # multi.beam_vs_thickness(iBs=range(10),cm='Spectral',name=figpath+'900_B.svg',opt='sp')
    # multi.beam_vs_thickness(tol=1e-3,cm='Spectral',name=figpath+'900_B.svg',opt='sp',xylims=['y',0,0.005])

# multi = pp.load_multi_obj('../dat/ireloh/ireloh_t1_autoslic.pkl')
multi = pp.load_multi_obj('../dat/ireloh/ireloh_484_autoslic.pkl')
# multi = pp.load_multi_obj('../dat/ireloh/ireloh_900_autoslic.pkl')
# multi = pp.load_multi_obj('../dat/ireloh/ireloh_t1_autoslic.pkl')
# multi = pp.load_multi_obj('../dat/ireloh/ireloh_pptest_autoslic.pkl')
# multi.postprocess(ssh_alias='badb',ppopt='uB')


# postpp(multi)



def plot_transmission_vs_structure_factor():
    plts=[]
    #### Structure factor
    if 'F' in opts:
        (qxF,qyF,qzF),Fhkl = multi.get_structure_factor(hkl=None, hklMax=hklM, sym=1, v='')
        If = np.abs(Fhkl)**2 # perfect Bragg condition and flat ewald sphere

        sF = np.s_[:,hklM,hklM] # slice in the [0,0,1] direction
        If[hklM,hklM,hklM] = 1e-6
        If/=If[hklM+2,hklM,hklM]
        plts+=[[qyF[sF],np.log10(If[sF]),'g-o','$F$']]
        # dsp.stddisp(im=[ h[s],k[s],np.log10(I[s])],xylims=[-hklM,hklM,-hklM,hklM],caxis=[0,5],
        #     imOpt='cv',axPos='V',cmap='binary')

    #### transmission function
    if 'T' in opts:
        qx,qy,It = multi.pattern(Iopt='Ics',out=True,tol=1e-3,gs=1.3,caxis=[-6.2,0],rings=[0.5,1,2],lw=2,
            imOpt='cv',axPos='V',cmap='binary',opt='c')
        # multi.pattern(Iopt='Incslg',out=False,tol=1e-3,gs=1.3,caxis=[-6.2,0],rings=[0.5,1,2],lw=2,
        #     imOpt='cv',axPos='V',cmap='binary',opt='p')
        Nx = int(qx.shape[0]/2)
        st = np.s_[:,Nx] # slice in the [0,0,1] direction
        It[Nx,Nx] = 1e-6
        It/=It[Nx+2,Nx]
        plts += [[qy[st],np.log10(It[st]),'b-o','$T_{cell}$']]

        ##small slice thickness
        multi1 = mupy.Multislice(path,data=name,tail='t1',
            mulslice=False,keV=200,#tilt=[tx*np.pi/180/1000,0],
            NxNy=1024,repeat=[1,1,1],slice_thick=1,Nhk=5,
            #TDS=True,T=300,n_TDS=15,
            opt='sr',fopt='',v='nctr',ppopt='w',#nctrdDR',
            #ssh=ssh,hostpath=hostpath
            )
        qx,qy,It1 = multi1.pattern(Iopt='Ics',out=True,tol=1e-3,gs=1.3,caxis=[-6.2,0],rings=[0.5,1,2],lw=2,
            imOpt='cv',axPos='V',cmap='binary',opt='c')
        # multi.pattern(Iopt='Incslg',out=False,tol=1e-3,gs=1.3,caxis=[-6.2,0],rings=[0.5,1,2],lw=2,
        #     imOpt='cv',axPos='V',cmap='binary',opt='p')
        Nx = int(qx.shape[0]/2)
        st = np.s_[:,Nx] # slice in the [0,0,1] direction
        It1[Nx,Nx] = 1e-6
        It1/=It1[Nx+2,Nx]
        plts += [[qy[st],np.log10(It1[st]),'c-o','$T_1$']]



        dsp.stddisp(plts,labs=['$q$','$I$'],xylims=[-2,2,-10,1])

# dsp.stddisp(im=[ h[s],k[s],np.log10(T[s])],xylims=[-hklM,hklM,-hklM,hklM],caxis=[0,5],
#     imOpt='cv',axPos='V',cmap='binary')



from utils import*
import importlib as imp
from utils import displayStandards as dsp
from crystals import Crystal
from multislice import multislice as mupy
from multislice import rotating_crystal as rcc; imp.reload(rcc)
from multislice import postprocess as pp


def Li_xyz(path,n=[0,0,1],dopt=1,lfact=1.0,wobble=0,tail=''):
    file    = path+'Li%s%s.xyz' %(''.join(np.array(n,dtype=str)),tail)
    crys    = Crystal.from_database('Li')
    pattern = np.array([
        [a.atomic_number]+list(lfact*a.coords_cartesian)+[a.occupancy,wobble] for a in crys.atoms])
    # lat_params = tuple(lfact*np.array(crys.lattice_parameters[:3]))
    lat_params = lfact*np.array(crys.lattice_vectors)
    coords,lat = rcc.make_xyz(file,pattern,lat_params,n,fmt='%.4f',dopt='s')
    return file

#########################################################################
def Li_latsize(path,nvals=2,**kwargs):
    df = pd.DataFrame(columns=['lat_cst','host','state']+info_cols)
    vals = np.array(2**np.arange(nvals),dtype=int) # np.linspace()
    plts0,plts1,plts2,cs1,Imax0,Imax1,Imax2 = [],[],[],getCs('viridis',nvals),0,0,0
    for i,val in zip(range(nvals),vals):
        tail = str(i).zfill(ceil(nvals/10))
        data = Li_xyz(path,n=[0,0,1],dopt=1,lfact=val,tail='_'+tail)
        multi=mupy.Multislice(path,tail=tail,data=data,
            mulslice=0,keV=200,
            NxNy=64*val,slice_thick=val*1.5,Nhk=5,repeat=[1,1,int(2**10/val)],
            **kwargs)
        df.loc[multi.outf['obj']] = [nan]*len(df.columns)
        df.loc[multi.outf['obj']]['lat_cst','host','state'] = [val,'','start']

        #hk,t,re,im,I = multi.get_beams(bOpt='f',tol=1e-4)
        hk,t,re,im,I = multi.get_beams(bOpt='f',iBs=['(1,1)','(2,0)','(4,0)'])
        Imax0,Imax1,Imax2 = max(Imax0,I[0].max()),max(Imax1,I[1].max()),max(Imax2,I[2].max())
        plts0 += [[t,I[0],[cs1[i],'-'],'$I_{%s}^{(%d)}$' %(hk[0],i)]]
        plts1 += [[t,I[1],[cs1[i],'-'],'$I_{%s}^{(%d)}$' %(hk[1],i)]]
        plts2 += [[t,I[2],[cs1[i],'-'],'$I_{%s}^{(%d)}$' %(hk[2],i)]]
    name,opt='docs_fig/lattice_effect','s'
    stddisp(plts0,labs=['$thickness$ ($A$)','$I_{beams}$'],
        xylims=[0,t.max(),0,Imax0],opt=opt,lw=2,axPos=1,setPos=1,name=name+'0.svg')
    stddisp(plts1,labs=['$thickness$ ($A$)','$I_{beams}$'],
        xylims=[0,t.max(),0,Imax1],opt=opt,lw=2,axPos=1,setPos=1,name=name+'1.svg')
    stddisp(plts2,labs=['$thickness$ ($A$)','$I_{beams}$'],
        xylims=[0,t.max(),0,Imax2],opt=opt,lw=2,axPos=1,setPos=1,name=name+'2.svg')
    df.to_pickle(path+'df.pkl')
    df=update_df_info(path+'df.pkl')
    return df


#########################################################################
def Li_patterns(name,nzs=20,**kwargs):
    param = 'repeat'
    reps  = np.arange(1,nzs+1)*5
    vals  = [[10,10,nz] for nz in reps]
    mupy.sweep_var(name,param,vals,df=1,do_prev=0,
        data='Li001.xyz',mulslice=0,keV=200,tail='',
        NxNy=512,slice_thick=1.5,Nhk=0,
        **kwargs)
def Li_gif(name):
    df=update_df_info(name+'df.pkl')
    nzs = df.index.size
    xt,yt = 0,4 #text position
    zI = df[['zmax(A)','Inorm']].values;#print(zI)
    for i in range(nzs):
        istr = str(i).zfill(ceil(nzs/10))
        tle = '$z=%.0f A, I_{norm}=%.4f$' %tuple(zI[i,:])
        multi=load_multi_obj(name+'Lithium_repeat%s_autoslic.pkl' %istr)
        multi.pattern(Iopt='Insc',tol=1e-3,Nmax=64,
            cmap='gray',texts=[xt,yt,tle,'g'],xylims=[-5,5,-5,5],
            axPos=[0,0,1,1],setPos=1,gridOn=0,ticksOn=False,
            name=name+'pattern%s.png' %istr,opt='s')
    #dsp.im2gif(name+'pattern','svg')

def Li_wobble(name,nvals=np.inf,**kwargs):
    wobbles = [0.001,0.01,0.1]
    nvals = min(nvals,len(wobbles))
    wobbles = np.array(wobbles)[:nvals]
    plts,cs = [],dsp.getCs('coolwarm',nvals)
    tol,Iopt = 1e-5,'Incsl',
    for i,val in zip(range(nvals),wobbles):
        tail = str(i).zfill(int(wobbles.size/10)+1)
        data = Li_xyz(name,n=[0,0,1],dopt=1,wobble=val,tail='_'+tail)
        multi=mupy.Multislice(name,tail=tail,data=data,
            mulslice=0,keV=100,TDS=True,n_TDS=20,
            NxNy=1024,slice_thick=1.5,Nhk=5,repeat=[8,8,100],
            **kwargs)
        q,I = multi.azim_avg(out=1,opt='',Iopt=Iopt,tol=tol)
        plts+=[[q,I,cs[i],'w=%.3f' %val]]
        multi.pattern(rings=[0.3,0.6,1,2],Iopt=Iopt,tol=tol,
            imOpt='ch',cmap='binary',axPos=[0.17,0.13,0.75,0.75],lw=2,xylims=[0,3,0,3],
            opt='sc',name='docs_fig/wobble_effect%d.png' %i,pOpt='tpX',
            caxis = [-5,0])

    dsp.stddisp(plts,labs=[r'$q(\AA^{-1})$','$I_q$'],lw=2,fonts={'leg':20},
        xylims=['x',0,4,-5,0],opt='sc',name='docs_fig/wobble_effectIavg.svg')

def Li_rots():
    crys = Crystal.from_database('Li')
    coords = np.array([a.coords_cartesian for a in crys.atoms])

    # files = rcc.rotate_xyz(crys,Nrot=8,name='dat/Lithium/rots/',opt='p')
    files = rcc.orient_crystal(coords,ez=[0,0,1],n_u=[0,,1])
    for file in files[:4] : rcc.show_grid(file,title=file,equal=1)#,xyTicks=3.49,xylims=[0,100,0,100])

##########################################################################
if __name__ == "__main__":
    plt.close('all')
    name = 'dat/Lithium/'
    # Li_xyz(name,n=[0,0,1],dopt=1)
    # Li_patterns(name+'gif/',nzs=20, opt='drsp',fopt='',ppopt='w',v=1)
    # Li_gif(name+'gif/')
    # df = Li_latsize(name+'latsize/',nvals=3,opt='drsp',fopt='',ppopt='w',v=1)
    # Li_wobble(name+'wobble/',nvals=3,opt='',fopt='',ppopt='wu',v=0)#,ssh='tarik-CCP4home')
    Li_rots()



from utils import*                  #;imp.reload(dsp)
from EDutils import viewers as vw   ;imp.reload(vw)
# from EDutils import viewer_config as vw_cfg       ;imp.reload(vw_cfg)

plt.close('all')

# v = vw.Viewer(path='dat',init_opts='',Nmax=5,tag='test',
#     pets_opts='PMS',frame=18,thick=24,rotS=70,rot=205)

v = vw.Viewer(config='dat/config.pkl')



from utils import*
import scipy.signal as sig
import importlib as imp
import wallpp.lattice as lat        ;imp.reload(lat)
import wallpp.plane_group as pg     ;imp.reload(pg)
import multislice.multi_2D as ms    ;imp.reload(ms)
plt.close('all')
path='../multislice/docs_fig/multi2D/'
opts='M'

keV   = 200
Nx,Nz = 1,50,#10,2  # unit cells
ndeg  = 2**5       # number of pixels
npx   = Nx*ndeg    # number of pixels
pptype,ax,bz,angle = 'p1',2,2,90
Za = 2
pattern = np.array([[1,1,1]])
Nh = 20

p1 = pg.Wallpaper(pptype,ax,bz,angle,pattern,ndeg=ndeg)
potential = p1.get_potential_grid()


# #### 2-beam setup
# rot = lambda t:np.array([[np.cos(t),np.sin(t)],[-np.sin(t),np.cos(t)]])
# a1,a2 = lat.get_lattice_vec(lat_type='rect',a=ax,b=bz)
# b1,b2 = lat.reciprocal_lattice_2D(a1,a2)
# Ai = np.array([0.1,0.25,0.26,0.27,1.5])
# fj = lambda ti,j:np.sqrt(np.pi*Ai[j])*np.exp(-(np.pi*Ai[j]*K*np.sin(ti))**2)
#
# n = 10
# x0,z0 = np.meshgrid(np.arange(-1,n),np.arange(n+1))
# x0,z0 = np.stack([x0*ax+pattern[0][0],z0*bz+pattern[0][1]])
# Xa = rot(1/n).dot(np.stack([x0.flatten(),z0.flatten()]))
#
# ax1 = np.sqrt(1**2+n**2)*ax
# bz1 = ax1
#
# x,z   = np.meshgrid(np.arange(n*ndeg)*ax1/(n*ndeg),np.arange(n*ndeg)*bz1/(n*ndeg))
# f = np.sum(np.array([pg.fv(np.stack([x.flatten(),z.flatten()]).T,X0,Za) for X0 in Xa.T]),axis=0)
# f = np.reshape(f,(n*ndeg,n*ndeg))
#
# if 'P' in opts:dsp.stddisp(im=[x,z,f],scat=[Xa[0],Xa[1],15,'k'],)

epss = np.arange(1,6)*0.01#array([0.01,0.05,0.1,0.2])
Nx,Nz = 1,2000
ms0 = []
for eps in epss:
    print(colors.red,eps,colors.black)
    ms0 += [ms.Multi2D(potential,ax,bz,keV=keV,Nx=Nx,nz=Nz,
            dz=bz/2,eps=eps,
            iZs=1,iZv=np.inf)]

plts,xi_g = [],np.zeros(epss.shape)
cs = dsp.getCs('hsv',epss.size)
for i,eps in enumerate(epss):
    Ib = ms0[i].Bz_show([1],out=1)[0]
    idx = sig.find_peaks(-Ib,width=5)[0][0]
    xi_g[i] = ms0[i].z[idx]
    plts+=[[ms0[i].z,-Ib,cs[i],r'$\epsilon=%.2f$' %eps]]
    plts+=[[xi_g[i],-Ib[idx],[cs[i],'s'],'']]

dsp.stddisp(plts,lw=2)
dsp.stddisp([epss,xi_g,'b-o'],lw=2)



from utils import*
plt.close('all')


df=pd.read_csv('dat/ex5_A1.csv') #Aluminum
ext = 'Extinc. / nm'

hs = np.arange(2,11,2)[:1]
s = np.linspace(-0.1,0.1,1000) # nm^-1
t = 130 #nm

xi=np.zeros(hs.shape)
for i,h in enumerate(hs):
    f = df[(df[' h ']==h) & (df['k ']==0) & (df['l ']==0)]
    xi_g =f[ext].values[0];#print(xi_g)
    xi[i] = xi_g
    Ig0 = 1/(1+xi_g**2*s**2)
    Ig = np.sin(np.pi*t*np.sqrt(1/xi_g**2+s**2))**2*Ig0
    plts = [[s,Ig,'k'],[s,Ig0,'r--']]
    dsp.stddisp(plts,xylims=['y',0,1.1*Ig.max()],title='(%d 0 0)' %h,labs=['$s(nm^{-1})$','$I_g$'],lw=2)



import setuptools

#with open("latex/README.md", "r") as fh:
#    long_description = fh.read()

setuptools.setup(
    name="tarikED",
    version="1.0.5c",
    author="Tarik Ronan Drevon",
    author_email="ronandrevon@gmail.com",
    description="Electron diffraction utilities",
    long_description='', #long_description,
    long_description_content_type="",
    url="",
    # packages=['multislice','scattering','wallpp'],
    package_data={'':['data/splines.npy','convert_pets.sh'],},
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License ",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['numpy','scipy','matplotlib','colorama','pandas',
    'cbf','crystals','TarikDrevonUtils','easygui','tifffile','pickle5'],
)



from utils import*
import multislice as mupy
#from postprocess import load_multi_obj as loadm
import postprocess as pp
from postprocess import hosts,cpus,ncpus,info_cols


def run_wallT(name,Nmax=7,Nslices=5,**kwargs):
    df_name = name+'/df.pkl'
    param,NxNys ='NxNy', 2**np.arange(5,Nmax)
    base_cols = [param,'host','state']
    #prepare dataframe and run
    df = pd.DataFrame(columns=base_cols+info_cols)
    for host in hosts:
        print(red+"HOST : " +host+black)
        mupy.sweep_var(name,param,NxNys,df,tail=host,mulslice=False,keV=200,
            repeat=[1,1,Nslices],slice_thick=1.375,Nhk=5,
            ssh=host,fopt='',**kwargs)
        df.to_pickle(df_name);print(green+'DataFrame saved : ' +yellow+df_name+black)
    return df

def postprocess_wallT(name,Nslices=1,opts='u',**kwargs):
    '''opts:
        - u(update), W (walltime else cputime),
        - c (per core), s(per slice) l(log_opt)'''
    df_name = name+'/df.pkl'
    df  = pd.read_pickle(df_name)
    lab = ['cputime','walltime']['W' in opts]
    tle,col,logopt = lab,lab +'(s)',''
    if 'u' in opts : df = pp.update_df_info(df_name)
    if 'c' in opts : lab,tle = lab+'_pcore' ,tle+' per core'
    if 's' in opts : lab,tle = lab+'_pslice',tle+' per slice'
    if 'l' in opts : lab,logopt =lab+'_log','xy'
    plots,cs=[],['r','g','b'] #,getCs('jet',len(hosts))
    for c_i,host in zip(cs,hosts):
        df_host = df.loc[df.host==host]
        NxNys = np.array(df_host['NxNy'].values,dtype=int)
        Ts = np.array(df_host[col].values,dtype=float)
        if 'c' in opts: Ts*=ncpus[host]
        if 's' in opts: Ts/=Nslices
        plots += [[NxNys**2,Ts,[c_i,'s-'],cpus[host]]]
    stddisp(plots,lw=2,name='docs_fig/autoslic_%s.svg' %(lab),logOpt=logopt,
        labs=['$N_{beams}$','time (s)'],
        title=tle,setPos=1,axPos='T',**kwargs)


########################################################################
if __name__ == "__main__":
    plt.close('all')
    name,Nxy_max,Nslices = 'dat/walltest',7,5
    #run_wallT(name,Nmax=Nxy_max,Nslices=Nslices,opt='dsr',v='nrR')
    postprocess_wallT(name,opts='ucls',Nslices=Nslices,opt='p')



from utils import*
import scipy.signal as sig
plt.close('all')


dfI=pd.read_csv('dat/ex5_2A_I.csv') #Silicon 2-beam intensities
dfS=pd.read_csv('dat/ex5_2A_Si_200kVdat.csv') #Silicon Simu
h,k,l,ext = ' h ','k ','l ','Extinc. / nm'
cols=[h,k,l,ext]
ds = 'Distance (nm)'
bf = 'Bright-field intensity (norm)'
df = 'Dark-field intensity (norm)'

t,Ib,Id = dfI[ds],dfI[bf],dfI[df]
plts = [[t,Ib,'r','$I_b$'],[t,Id,'b','$I_d$']]


idx = sig.find_peaks(-Ib,width=10)[0]
# idx = sig.find_peaks(-Id,width=10)[0]
plts += [[t[idx],Ib[idx],'gs']]
# plts += [[t[idx],Id[idx],'gs']]
dsp.stddisp(plts,labs=['$t(nm)$','$I$'],lw=2)

xi = np.diff(t[idx]);print(xi)

print(dfS[cols])



from utils import*                      ;imp.reload(dsp)
from EDutils import viewers as vw       ;imp.reload(vw)
from multislice import postprocess as pp#;imp.reload(pp)
from multislice import multislice as ms ;imp.reload(ms)
from blochwave import bloch as bl       ;imp.reload(bl)

plt.close('all')
opts = 'vb'
path = 'dat/'

# v = vw.Viewer(path='dat',cif_file='diamond',init_opts='R')
if 'v' in opts:
    v = vw.Viewer(config=path+'config.pkl')



if 'b' in opts:
    tag='001'
    multi = pp.load(path+'multi',tag+'001')
    fig,ax = multi.beam_vs_thickness(bOpt='',tol=0.001,opt='')
    # fig,ax = multi.beam_vs_thickness(tol=1e-3,opt='')
    bloch = bl.load_Bloch(path+'bloch/',tag+'0001')
    bloch.solve()
    bloch.set_beams_vs_thickness((0,2000,1000))#bloch.solve()
    bloch.show_beams_vs_thickness(strong=['Ig'],ax=ax,linespec='--',xylims=['x',0,300])

if 'i' in opts:
    multi = pp.load(path+'multi','2beams001')
    idx = np.array([[512,613],[512,512]])
    args = {'Iopt':'csn','Imax':5e4,'gs':0.025,'rmax':25,'Nmax':512}
    multi.integrate_reflections(idx,N=5,**args)



from utils import*
from postprocess import*
from utils import displayStandards as dsp
from utils.handler3D import handler_3d
from crystals import Crystal
from wallpp import lattice
import multislice as mupy
import sys,numpy,os
import rotating_crystal as rcc
#sys.backtrace=0
cs = {1:unicolor(0.75),6:unicolor(0.25),7:(0,0,1),8:(1,0,0)}

#########
def show_grid(name):
    pattern,crys = APAP_xyz(name,n=[0,0,1],dopt=0)
    a1,a2,a3,alpha,beta,gamma=crys.lattice_parameters
    a,b,angle=a3,a1,beta
    Z,x,y,z = pattern[:,:4].T
    C = [cs[z] for z in Z]
    fig,ax = stddisp(scat=[z,x,C],ms=50,opt='')
    lat_vec = lattice.get_lattice_vec('oblique',a=a,b=b,alpha=angle,v=0)
    lattice.plot_lattice(lat_vec,opts='',nh=3,nk=3,
        ax=ax,labs=['$z$','$x$'],lw=2,
        pOpt='tG',pad=1,xyTicks=[1.5,5.0],xylims=[-6,15,0,21])#,xyTicksm=1.0,xylims=[0,100,0,100])

def get_unit_cell(cell_mesh,n):
    alpha,lw,c = 0.1,2,'c'
    x,y,z = cell_mesh
    planes_coords,ncells = [], x.shape[0]
    for i in range(ncells):
        planes_coords+=[[x[i,:,:],y[i,:,:],z[i,:,:]],
                        [x[:,i,:],y[:,i,:],z[:,i,:]],
                        [x[:,:,i],y[:,:,i],z[:,:,i]]]
    surfs = []
    for plane_coords in planes_coords :
        x,y,z  = plane_coords
        p_shape = x.shape
        coords = np.array([x.flatten(),y.flatten(),z.flatten()]);#print(coords.shape)
        x,y,z  = rcc.orient_crystal(coords,n_u=n,T=False);
        x,y,z  = np.reshape(x,p_shape),np.reshape(y,p_shape),np.reshape(z,p_shape)
        surfs += [[x,y,z,c,alpha,lw,c]]
    return surfs

def show_cell(file,n=[0,0,1],bopt=1,x0=[0,0,0],rep=[1,1,1],**kwargs):
    crys = Crystal.from_cif(file)
    tail = ''.join(np.array(n,dtype=str))
    n    = get_vec(n,crys,bopt)

    #unit cells,lattice vectors,atom coordinates
    cell_mesh   = crys.mesh(range(rep[0]+1),range(rep[1]+1),range(rep[2]+1))
    surfs       = get_unit_cell(cell_mesh,n)
    uvw         = rcc.orient_crystal(np.array(crys.lattice_vectors),n_u=n,T=True)
    pattern,lat_params = APAP_xyz(name,n,rep,dopt='s',tail=tail)
    E,X,Y,Z     = pattern[:,:4].T
    scat        = [X,Y,Z, [cs[int(e)] for e in E]]

    scat2=[]
    # pattern2,lat_params = APAP_xyz(name,n,rep,dopt='')
    # E2,X2,Y2,Z2 = pattern2[:,:4].T
    # scat2       = [X2,Y2,Z2, [cs[int(e)] for e in E2]];#print(E2)

    #display options
    c1,c2,c3 = (X.min()+X.max())/2,(Y.min()+Y.max())/2,(Z.min()+Z.max())/2
    w = 0.75*max(X.max()-X.min(),Y.max()-Y.min(),Z.max()-Z.min())
    xylims=[c1-w/2,c1+w/2,c2-w/2,c2+w/2,c3-w/2,c3+w/2]
    fig,ax=dsp.stddisp(scat=scat,ms=100,surfs=surfs,rc='3d',std=0)
    rcc.show_trihedron(ax,uvw=uvw,x0=[0,0,0],cs=['r','g','b'],labs=['$a$','$b$','$c$'],lw=2,rc=0.1,h=0.2)
    rcc.show_trihedron(ax,x0=x0,labs=['$x$','$y$','$z$'],lw=2,rc=0.1,h=0.2)
    dsp.stddisp(ax=ax,ms=50,scat=scat2,xylims=xylims,axPos=[0,0,1,1],pOpt='eXp',**kwargs)

    hdl = handler_3d(fig,persp=False)



def APAP_xyz(name,n=[0,0,1],rep=[1,1,1],dopt='scp',lfact=1.0,tail=''):
    path    = os.path.dirname(name) + '/'
    crys    = Crystal.from_cif(path+'paracetamol.cif')
    lat_vec = np.array(crys.lattice_vectors)
    if sum(rep)>3 : crys = crys.supercell(rep[0],rep[1],rep[2])
    pattern = np.array([[a.atomic_number]+list(lfact*a.coords_cartesian)+[a.occupancy,1.0] for a in crys.atoms])
    file   = path+'APAP%s.xyz' %tail
    pattern,lat_params = mupy.make_xyz(file,pattern,lat_vec,n,fmt='%.4f',dopt=dopt)
    return pattern,lat_params #pattern,crys # file

def get_vec(n,crys,bopt) :
    if isinstance(n,int) : n = crys.lattice_vectors[n]
    if bopt : n = np.array(n).dot(np.array(crys.lattice_vectors))
    return n

##########################################################################
if __name__ == "__main__":
    plt.close('all')
    name = 'dat/Paracetamol/paracetamol.cif'
    # p0=APAP_xyz(name,n=0,dopt=1,lfact=1.0,tail='')
    # p2=APAP_xyz(name,n=2,dopt=1,lfact=1.0,tail='')
    show_cell(name,n=[1,0,0],bopt=1,rep=[2,1,2]) #z along a
    # show_cell(name,n=[0,0,1],bopt=1) #z along c
    # show_cell(name,n=[1,0,1],bopt=1,rep=[2,2,2]) #z along a+c
    # show_cell(name,n=0,x0=[3,0,-3] ,view=[ 0,90] ,name='docs_fig/APAP100_xz.png',opt='p')
    # show_cell(name,n=0,x0=[3,0,-3] ,view=[90,90] ,name='docs_fig/APAP100_xy.png',opt='p')
    # show_cell(name,n=0,x0=[3,0,-3] ,view=[30,70] ,name='docs_fig/APAP100_3D.png',opt='p')
    # show_cell(name,n=2,x0=[-1,0,-3],view=[ 0,90] ,name='docs_fig/APAP001_xz.png',opt='p')
    # show_cell(name,n=2,x0=[-2,-2,0],view=[90,90] ,name='docs_fig/APAP001_xy.png',opt='p')
    # show_cell(name,n=2,x0=[3,-5,-3],view=[25,-65],name='docs_fig/APAP001_3D.png',opt='p')
    #show_grid(name)
